<?php

namespace Boomee\GoodToKnow\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Boomee\GoodToKnow\Http\Requests\NewsRequest;
use Boomee\GoodToKnow\Interfaces\NewsRepositoryInterface;
use Boomee\GoodToKnow\Models\News;

/**
 * Resource controller class for news.
 */
class NewsResourceController extends BaseController
{

    /**
     * Initialize news resource controller.
     *
     * @param type NewsRepositoryInterface $news
     *
     * @return null
     */
    public function __construct(NewsRepositoryInterface $news)
    {
        parent::__construct();
        $this->repository = $news;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Boomee\GoodToKnow\Repositories\Criteria\NewsResourceCriteria::class);
    }

    /**
     * Display a list of news.
     *
     * @return Response
     */
    public function index(NewsRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Boomee\GoodToKnow\Repositories\Presenter\NewsPresenter::class)
                ->$function();
        }

        $news = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('good_to__know::news.names'))
            ->view('good_to__know::news.index', true)
            ->data(compact('news', 'view'))
            ->output();
    }

    /**
     * Display news.
     *
     * @param Request $request
     * @param Model   $news
     *
     * @return Response
     */
    public function show(NewsRequest $request, News $news)
    {

        if ($news->exists) {
            $view = 'good_to__know::news.show';
        } else {
            $view = 'good_to__know::news.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('good_to__know::news.name'))
            ->data(compact('news'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new news.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(NewsRequest $request)
    {

        $news = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('good_to__know::news.name')) 
            ->view('good_to__know::news.create', true) 
            ->data(compact('news'))
            ->output();
    }

    /**
     * Create new news.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(NewsRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $news                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('good_to__know::news.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('good_to__know/news/' . $news->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/good_to__know/news'))
                ->redirect();
        }

    }

    /**
     * Show news for editing.
     *
     * @param Request $request
     * @param Model   $news
     *
     * @return Response
     */
    public function edit(NewsRequest $request, News $news)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('good_to__know::news.name'))
            ->view('good_to__know::news.edit', true)
            ->data(compact('news'))
            ->output();
    }

    /**
     * Update the news.
     *
     * @param Request $request
     * @param Model   $news
     *
     * @return Response
     */
    public function update(NewsRequest $request, News $news)
    {
        try {
            $attributes = $request->all();

            $news->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('good_to__know::news.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('good_to__know/news/' . $news->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('good_to__know/news/' . $news->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the news.
     *
     * @param Model   $news
     *
     * @return Response
     */
    public function destroy(NewsRequest $request, News $news)
    {
        try {

            $news->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('good_to__know::news.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('good_to__know/news/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('good_to__know/news/' . $news->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple news.
     *
     * @param Model   $news
     *
     * @return Response
     */
    public function delete(NewsRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('good_to__know::news.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('good_to__know/news'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/good_to__know/news'))
                ->redirect();
        }

    }

    /**
     * Restore deleted news.
     *
     * @param Model   $news
     *
     * @return Response
     */
    public function restore(NewsRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('good_to__know::news.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/good_to__know/news'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/good_to__know/news/'))
                ->redirect();
        }

    }

}
